@@ -46,6 +46,7 @@
 #include "eloop.h"
 #include "if.h"
 #include "logerr.h"
+#include "privsep.h"
 
 #ifndef SUN_LEN
 #define SUN_LEN(su) \
@@ -63,7 +64,6 @@ control_queue_free(struct fd_list *fd)
 			free(fdp->data);
 		free(fdp);
 	}
-	fd->queue_len = 0;
 
 #ifdef CTL_FREE_LIST
 	while ((fdp = TAILQ_FIRST(&fd->free_queue))) {
@@ -75,56 +75,118 @@ control_queue_free(struct fd_list *fd)
 #endif
 }
 
-static void
+void
+control_free(struct fd_list *fd)
+{
+
+#ifdef PRIVSEP
+	if (fd->ctx->ps_control_client == fd)
+		fd->ctx->ps_control_client = NULL;
+#endif
+
+	eloop_event_remove_writecb(fd->ctx->eloop, fd->fd);
+	TAILQ_REMOVE(&fd->ctx->control_fds, fd, next);
+	control_queue_free(fd);
+	free(fd);
+}
+
+void
 control_delete(struct fd_list *fd)
 {
 
-	TAILQ_REMOVE(&fd->ctx->control_fds, fd, next);
+#ifdef PRIVSEP
+	if (IN_PRIVSEP_SE(fd->ctx))
+		return;
+#endif
+
 	eloop_event_delete(fd->ctx->eloop, fd->fd);
 	close(fd->fd);
-	control_queue_free(fd);
-	free(fd);
+	control_free(fd);
 }
 
 static void
 control_handle_data(void *arg)
 {
 	struct fd_list *fd = arg;
-	char buffer[1024], *e, *p, *argvp[255], **ap, *a;
+	char buffer[1024];
 	ssize_t bytes;
-	size_t len;
-	int argc;
 
 	bytes = read(fd->fd, buffer, sizeof(buffer) - 1);
+
 	if (bytes == -1 || bytes == 0) {
 		/* Control was closed or there was an error.
 		 * Remove it from our list. */
 		control_delete(fd);
 		return;
 	}
-	buffer[bytes] = '\0';
-	p = buffer;
-	e = buffer + bytes;
+
+#ifdef PRIVSEP
+	if (IN_PRIVSEP(fd->ctx)) {
+		ssize_t err;
+
+		fd->flags |= FD_SENDLEN;
+		err = ps_ctl_handleargs(fd, buffer, (size_t)bytes);
+		fd->flags &= ~FD_SENDLEN;
+		if (err == -1) {
+			logerr(__func__);
+			return;
+		}
+		if (err == 1 &&
+		    ps_ctl_sendargs(fd, buffer, (size_t)bytes) == -1) {
+			logerr(__func__);
+			control_delete(fd);
+		}
+		return;
+	}
+#endif
+
+	control_recvdata(fd, buffer, (size_t)bytes);
+}
+
+void
+control_recvdata(struct fd_list *fd, char *data, size_t len)
+{
+	char *p = data, *e;
+	char *argvp[255], **ap;
+	int argc;
 
 	/* Each command is \n terminated
 	 * Each argument is NULL separated */
-	while (p < e) {
+	while (len != 0) {
 		argc = 0;
 		ap = argvp;
-		while (p < e) {
-			argc++;
-			if ((size_t)argc >= sizeof(argvp) / sizeof(argvp[0])) {
-				errno = ENOBUFS;
+		while (len != 0) {
+			if (*p == '\0') {
+				p++;
+				len--;
+				continue;
+			}
+			e = memchr(p, '\0', len);
+			if (e == NULL) {
+				errno = EINVAL;
+				logerrx("%s: no terminator", __func__);
 				return;
 			}
-			a = *ap++ = p;
-			len = strlen(p);
-			p += len + 1;
-			if (len && a[len - 1] == '\n') {
-				a[len - 1] = '\0';
+			if ((size_t)argc >= sizeof(argvp) / sizeof(argvp[0])) {
+				errno = ENOBUFS;
+				logerrx("%s: no arg buffer", __func__);
+				return;
+			}
+			*ap++ = p;
+			argc++;
+			e++;
+			len -= (size_t)(e - p);
+			p = e;
+			e--;
+			if (*(--e) == '\n') {
+				*e = '\0';
 				break;
 			}
 		}
+		if (argc == 0) {
+			logerrx("%s: no args", __func__);
+			continue;
+		}
 		*ap = NULL;
 		if (dhcpcd_handleargs(fd->ctx, fd, argc, argvp) == -1) {
 			logerr(__func__);
@@ -136,6 +198,26 @@ control_handle_data(void *arg)
 	}
 }
 
+struct fd_list *
+control_new(struct dhcpcd_ctx *ctx, int fd, unsigned int flags)
+{
+	struct fd_list *l;
+
+	l = malloc(sizeof(*l));
+	if (l == NULL)
+		return NULL;
+
+	l->ctx = ctx;
+	l->fd = fd;
+	l->flags = flags;
+	TAILQ_INIT(&l->queue);
+#ifdef CTL_FREE_LIST
+	TAILQ_INIT(&l->free_queue);
+#endif
+	TAILQ_INSERT_TAIL(&ctx->control_fds, l, next);
+	return l;
+}
+
 static void
 control_handle1(struct dhcpcd_ctx *ctx, int lfd, unsigned int fd_flags)
 {
@@ -154,20 +236,19 @@ control_handle1(struct dhcpcd_ctx *ctx, int lfd, unsigned int fd_flags)
 	    fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1)
 		goto error;
 
-	l = malloc(sizeof(*l));
+#ifdef PRIVSEP
+	if (IN_PRIVSEP(ctx) && !IN_PRIVSEP_SE(ctx))
+		;
+	else
+#endif
+	fd_flags |= FD_SENDLEN;
+
+	l = control_new(ctx, fd, fd_flags);
 	if (l == NULL)
 		goto error;
 
-	l->ctx = ctx;
-	l->fd = fd;
-	l->flags = fd_flags;
-	TAILQ_INIT(&l->queue);
-	l->queue_len = 0;
-#ifdef CTL_FREE_LIST
-	TAILQ_INIT(&l->free_queue);
-#endif
-	TAILQ_INSERT_TAIL(&ctx->control_fds, l, next);
-	eloop_event_add(ctx->eloop, l->fd, control_handle_data, l);
+	if (eloop_event_add(ctx->eloop, l->fd, control_handle_data, l) == -1)
+		logerr(__func__);
 	return;
 
 error:
@@ -192,6 +273,26 @@ control_handle_unpriv(void *arg)
 	control_handle1(ctx, ctx->control_unpriv_fd, FD_UNPRIV);
 }
 
+static int
+make_path(char *path, size_t len, const char *ifname, sa_family_t family)
+{
+	const char *per;
+
+	switch(family) {
+	case AF_INET:
+		per = "-4";
+		break;
+	case AF_INET6:
+		per = "-6";
+		break;
+	default:
+		per = "";
+		break;
+	}
+	return snprintf(path, len, CONTROLSOCKET,
+	    ifname ? ifname : "", ifname ? per : "", ifname ? "." : "");
+}
+
 static int
 make_sock(struct sockaddr_un *sa, const char *ifname, sa_family_t family,
     bool unpriv)
@@ -204,23 +305,8 @@ make_sock(struct sockaddr_un *sa, const char *ifname, sa_family_t family,
 	sa->sun_family = AF_UNIX;
 	if (unpriv)
 		strlcpy(sa->sun_path, UNPRIVSOCKET, sizeof(sa->sun_path));
-	else {
-		const char *per;
-
-		switch(family) {
-		case AF_INET:
-			per = "-4";
-			break;
-		case AF_INET6:
-			per = "-6";
-			break;
-		default:
-			per = "";
-			break;
-		}
-		snprintf(sa->sun_path, sizeof(sa->sun_path), CONTROLSOCKET,
-		    ifname ? ifname : "", ifname ? per : "", ifname ? "." : "");
-	}
+	else
+		make_path(sa->sun_path, sizeof(sa->sun_path), ifname, family);
 	return fd;
 }
 
@@ -252,6 +338,14 @@ control_start1(struct dhcpcd_ctx *ctx, const char *ifname, sa_family_t family,
 		return -1;
 	}
 
+#ifdef PRIVSEP_RIGHTS
+	if (IN_PRIVSEP(ctx) && ps_rights_limit_fd_fctnl(fd) == -1) {
+		close(fd);
+		unlink(sa.sun_path);
+		return -1;
+	}
+#endif
+
 	if ((fmode & S_UNPRIV) != S_UNPRIV)
 		strlcpy(ctx->control_sock, sa.sun_path,
 		    sizeof(ctx->control_sock));
@@ -263,6 +357,14 @@ control_start(struct dhcpcd_ctx *ctx, const char *ifname, sa_family_t family)
 {
 	int fd;
 
+#ifdef PRIVSEP
+	if (IN_PRIVSEP_SE(ctx)) {
+		make_path(ctx->control_sock, sizeof(ctx->control_sock),
+		    ifname, family);
+		return 0;
+	}
+#endif
+
 	if ((fd = control_start1(ctx, ifname, family, S_PRIV)) == -1)
 		return -1;
 
@@ -287,7 +389,7 @@ control_unlink(struct dhcpcd_ctx *ctx, const char *file)
 
 	errno = 0;
 #ifdef PRIVSEP
-	if (ctx->options & DHCPCD_PRIVSEP)
+	if (IN_PRIVSEP(ctx))
 		retval = (int)ps_root_unlink(ctx, file);
 	else
 #else
@@ -304,8 +406,21 @@ control_stop(struct dhcpcd_ctx *ctx)
 	int retval = 0;
 	struct fd_list *l;
 
-	if (ctx->options & DHCPCD_FORKED)
-		return 0;
+	while ((l = TAILQ_FIRST(&ctx->control_fds)) != NULL) {
+		control_free(l);
+	}
+
+#ifdef PRIVSEP
+	if (IN_PRIVSEP_SE(ctx)) {
+		if (ps_root_unlink(ctx, ctx->control_sock) == -1)
+			retval = -1;
+		if (ctx->options & DHCPCD_MASTER &&
+		    control_unlink(ctx, UNPRIVSOCKET) == -1)
+			retval = -1;
+		return retval;
+	} else if (ctx->options & DHCPCD_FORKED)
+		return retval;
+#endif
 
 	if (ctx->control_fd != -1) {
 		eloop_event_delete(ctx->eloop, ctx->control_fd);
@@ -323,14 +438,6 @@ control_stop(struct dhcpcd_ctx *ctx)
 			retval = -1;
 	}
 
-	while ((l = TAILQ_FIRST(&ctx->control_fds))) {
-		TAILQ_REMOVE(&ctx->control_fds, l, next);
-		eloop_event_delete(ctx->eloop, l->fd);
-		close(l->fd);
-		control_queue_free(l);
-		free(l);
-	}
-
 	return retval;
 }
 
@@ -381,23 +488,31 @@ control_writeone(void *arg)
 {
 	struct fd_list *fd;
 	struct iovec iov[2];
+	int iov_len;
 	struct fd_data *data;
 
 	fd = arg;
 	data = TAILQ_FIRST(&fd->queue);
-	iov[0].iov_base = &data->data_len;
-	iov[0].iov_len = sizeof(size_t);
-	iov[1].iov_base = data->data;
-	iov[1].iov_len = data->data_len;
-	if (writev(fd->fd, iov, 2) == -1) {
-		logerr(__func__);
-		if (errno != EINTR && errno != EAGAIN)
-			control_delete(fd);
+
+	if (data->data_flags & FD_SENDLEN) {
+		iov[0].iov_base = &data->data_len;
+		iov[0].iov_len = sizeof(size_t);
+		iov[1].iov_base = data->data;
+		iov[1].iov_len = data->data_len;
+		iov_len = 2;
+	} else {
+		iov[0].iov_base = data->data;
+		iov[0].iov_len = data->data_len;
+		iov_len = 1;
+	}
+
+	if (writev(fd->fd, iov, iov_len) == -1) {
+		logerr("%s: write", __func__);
+		control_delete(fd);
 		return;
 	}
 
 	TAILQ_REMOVE(&fd->queue, data, next);
-	fd->queue_len--;
 #ifdef CTL_FREE_LIST
 	TAILQ_INSERT_TAIL(&fd->free_queue, data, next);
 #else
@@ -406,30 +521,34 @@ control_writeone(void *arg)
 	free(data);
 #endif
 
-	if (TAILQ_FIRST(&fd->queue) == NULL)
-		eloop_event_remove_writecb(fd->ctx->eloop, fd->fd);
+	if (TAILQ_FIRST(&fd->queue) != NULL)
+		return;
+
+	eloop_event_remove_writecb(fd->ctx->eloop, fd->fd);
+#ifdef PRIVSEP
+	if (IN_PRIVSEP_SE(fd->ctx) && !(fd->flags & FD_LISTEN)) {
+		if (ps_ctl_sendeof(fd) == -1)
+			logerr(__func__);
+		control_free(fd);
+	}
+#endif
 }
 
 int
-control_queue(struct fd_list *fd, void *data, size_t data_len, bool fit)
+control_queue(struct fd_list *fd, void *data, size_t data_len)
 {
 	struct fd_data *d;
 
-	if (data_len == 0)
-		return 0;
+	if (data_len == 0) {
+		errno = EINVAL;
+		return -1;
+	}
 
 #ifdef CTL_FREE_LIST
 	struct fd_data *df;
 
 	d = NULL;
 	TAILQ_FOREACH(df, &fd->free_queue, next) {
-		if (!fit) {
-			if (df->data_size == 0) {
-				d = df;
-				break;
-			}
-			continue;
-		}
 		if (d == NULL || d->data_size < df->data_size) {
 			d = df;
 			if (d->data_size <= data_len)
@@ -441,28 +560,11 @@ control_queue(struct fd_list *fd, void *data, size_t data_len, bool fit)
 	else
 #endif
 	{
-		if (fd->queue_len == CONTROL_QUEUE_MAX) {
-			errno = ENOBUFS;
-			return -1;
-		}
-		fd->queue_len++;
 		d = calloc(1, sizeof(*d));
 		if (d == NULL)
 			return -1;
 	}
 
-	if (!fit) {
-#ifdef CTL_FREE_LIST
-		if (d->data_size != 0) {
-			free(d->data);
-			d->data_size = 0;
-		}
-#endif
-		d->data = data;
-		d->data_len = data_len;
-		goto queue;
-	}
-
 	if (d->data_size == 0)
 		d->data = NULL;
 	if (d->data_size < data_len) {
@@ -477,8 +579,8 @@ control_queue(struct fd_list *fd, void *data, size_t data_len, bool fit)
 	}
 	memcpy(d->data, data, data_len);
 	d->data_len = data_len;
+	d->data_flags = fd->flags & FD_SENDLEN;
 
-queue:
 	TAILQ_INSERT_TAIL(&fd->queue, d, next);
 	eloop_event_add_w(fd->ctx->eloop, fd->fd, control_writeone, fd);
 	return 0;
