@@ -117,8 +117,11 @@ struct passwd;
 struct dhcpcd_ctx {
 	char pidfile[sizeof(PIDFILE) + IF_NAMESIZE + 1];
 	char vendor[256];
-	int fork_fd;	/* FD for the fork init signal pipe */
+	bool stdin_valid;	/* It's possible stdin, stdout and stderr */
+	bool stdout_valid;	/* could be closed when dhcpcd starts. */
+	bool stderr_valid;
 	int stderr_fd;	/* FD for logging to stderr */
+	int fork_fd;	/* FD for the fork init signal pipe */
 	const char *cffile;
 	unsigned long long options;
 	char *logfile;
@@ -132,6 +135,7 @@ struct dhcpcd_ctx {
 	char **ifv;	/* listed interfaces */
 	int ifcc;	/* configured interfaces */
 	char **ifcv;	/* configured interfaces */
+	uint8_t duid_type;
 	unsigned char *duid;
 	size_t duid_len;
 	struct if_head *ifaces;
@@ -267,7 +271,7 @@ void dhcpcd_daemonise(struct dhcpcd_ctx *);
 
 void dhcpcd_linkoverflow(struct dhcpcd_ctx *);
 int dhcpcd_handleargs(struct dhcpcd_ctx *, struct fd_list *, int, char **);
-void dhcpcd_handlecarrier(struct dhcpcd_ctx *, int, unsigned int, const char *);
+void dhcpcd_handlecarrier(struct interface *, int, unsigned int);
 int dhcpcd_handleinterface(void *, int, const char *);
 void dhcpcd_handlehwaddr(struct interface *, uint16_t, const void *, uint8_t);
 void dhcpcd_dropinterface(struct interface *, const char *);
