@@ -60,6 +60,7 @@
 #include <linux/if_arp.h>
 #endif
 
+#include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <ctype.h>
@@ -362,7 +363,7 @@ if_vlanid(const struct interface *ifp)
 	memset(&v, 0, sizeof(v));
 	strlcpy(v.device1, ifp->name, sizeof(v.device1));
 	v.cmd = GET_VLAN_VID_CMD;
-	if (ioctl(ifp->ctx->pf_inet_fd, SIOCSIFVLAN, &v) != 0)
+	if (ioctl(ifp->ctx->pf_inet_fd, SIOCGIFVLAN, &v) != 0)
 		return 0; /* 0 means no VLANID */
 	return (unsigned short)v.u.VID;
 }
@@ -383,6 +384,48 @@ if_linksocket(struct sockaddr_nl *nl, int protocol, int flags)
 	return fd;
 }
 
+char *
+if_getnetworknamespace(char *buf, size_t len)
+{
+	struct stat sb_self, sb_netns;
+	DIR *dir;
+	struct dirent *de;
+	char file[PATH_MAX], *bufp = NULL;
+
+	if (stat("/proc/self/ns/net", &sb_self) == -1)
+		return NULL;
+
+	dir = opendir("/var/run/netns");
+	if (dir == NULL)
+		return NULL;
+
+	while ((de = readdir(dir)) != NULL) {
+		snprintf(file, sizeof(file), "/var/run/netns/%s", de->d_name);
+		if (stat(file, &sb_netns) == -1)
+			continue;
+		if (sb_self.st_dev != sb_netns.st_dev &&
+		    sb_self.st_ino != sb_netns.st_ino)
+			continue;
+		strlcpy(buf, de->d_name, len);
+		bufp = buf;
+		break;
+	}
+	closedir(dir);
+	return bufp;
+}
+
+int
+os_init(void)
+{
+	char netns[PATH_MAX], *p;
+
+	p = if_getnetworknamespace(netns, sizeof(netns));
+	if (p != NULL)
+		loginfox("network namespace: %s", p);
+
+	return 0;
+}
+
 int
 if_opensockets_os(struct dhcpcd_ctx *ctx)
 {
@@ -466,11 +509,9 @@ if_setmac(struct interface *ifp, void *mac, uint8_t maclen)
 }
 
 int
-if_carrier(struct interface *ifp)
+if_carrier(struct interface *ifp, __unused const void *ifadata)
 {
 
-	if (if_getflags(ifp) == -1)
-		return LINK_UNKNOWN;
 	return ifp->flags & IFF_RUNNING ? LINK_UP : LINK_DOWN;
 }
 
@@ -968,9 +1009,9 @@ link_netlink(struct dhcpcd_ctx *ctx, void *arg, struct nlmsghdr *nlm)
 		dhcpcd_handlehwaddr(ifp, ifi->ifi_type, hwa, hwl);
 	}
 
-	dhcpcd_handlecarrier(ctx,
+	dhcpcd_handlecarrier(ifp,
 	    ifi->ifi_flags & IFF_RUNNING ? LINK_UP : LINK_DOWN,
-	    ifi->ifi_flags, ifn);
+	    ifi->ifi_flags);
 	return 0;
 }
 
@@ -1614,7 +1655,6 @@ if_initrt(struct dhcpcd_ctx *ctx, rb_tree_t *kroutes, int af)
 	    &_if_initrt, kroutes);
 }
 
-
 #ifdef INET
 /* Linux is a special snowflake when it comes to BPF. */
 const char *bpf_name = "Packet Socket";
