@@ -185,7 +185,11 @@ if_setflag(struct interface *ifp, short setflag, short unsetflag)
 	    if_ioctl(ifp->ctx, SIOCSIFFLAGS, &ifr, sizeof(ifr)) == -1)
 		return -1;
 
-	ifp->flags = (unsigned int)ifr.ifr_flags;
+	/*
+	 * Do NOT set ifp->flags here.
+	 * We need to listen for flag updates from the kernel as they
+	 * need to sync with carrier.
+	 */
 	return 0;
 }
 
@@ -399,7 +403,7 @@ if_check_arphrd(struct interface *ifp, unsigned int active, bool if_noconf)
 		break;
 	case ARPHRD_LOOPBACK:
 	case ARPHRD_PPP:
-		if (if_noconf) {
+		if (if_noconf && active) {
 			logdebugx("%s: ignoring due to interface type and"
 			    " no config",
 			    ifp->name);
@@ -519,8 +523,11 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 
 #ifdef PLUGIN_DEV
 		/* Ensure that the interface name has settled */
-		if (!dev_initialized(ctx, spec.devname))
+		if (!dev_initialised(ctx, spec.devname)) {
+			logdebugx("%s: waiting for interface to initialise",
+			    spec.devname);
 			continue;
+		}
 #endif
 
 		if (if_vimaster(ctx, spec.devname) == 1) {
@@ -535,8 +542,11 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 		    !if_hasconf(ctx, spec.devname));
 
 		/* Don't allow some reserved interface names unless explicit. */
-		if (if_noconf && if_ignore(ctx, spec.devname))
+		if (if_noconf && if_ignore(ctx, spec.devname)) {
+			logdebugx("%s: ignoring due to interface type and"
+			    " no config", spec.devname);
 			active = IF_INACTIVE;
+		}
 
 		ifp = calloc(1, sizeof(*ifp));
 		if (ifp == NULL) {
@@ -581,7 +591,7 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 			case IFT_LOOP: /* FALLTHROUGH */
 			case IFT_PPP:
 				/* Don't allow unless explicit */
-				if (if_noconf) {
+				if (if_noconf && active) {
 					logdebugx("%s: ignoring due to"
 					    " interface type and"
 					    " no config",
@@ -681,40 +691,13 @@ if_discover(struct dhcpcd_ctx *ctx, struct ifaddrs **ifaddrs,
 #endif
 
 		ifp->active = active;
-		if (ifp->active)
-			ifp->carrier = if_carrier(ifp);
-		else
-			ifp->carrier = LINK_UNKNOWN;
+		ifp->carrier = if_carrier(ifp, ifa->ifa_data);
 		TAILQ_INSERT_TAIL(ifs, ifp, next);
 	}
 
 	return ifs;
 }
 
-static void
-if_poll(void *arg)
-{
-	struct interface *ifp = arg;
-	unsigned int flags = ifp->flags;
-	int carrier;
-
-	carrier = if_carrier(ifp); /* if_carrier will update ifp->flags */
-	if (ifp->carrier != carrier || ifp->flags != flags)
-		dhcpcd_handlecarrier(ifp->ctx, carrier, ifp->flags, ifp->name);
-
-	if (ifp->options->poll != 0 || ifp->carrier != LINK_UP)
-		if_pollinit(ifp);
-}
-
-int
-if_pollinit(struct interface *ifp)
-{
-	unsigned long msec;
-
-	msec = ifp->options->poll != 0 ? ifp->options->poll : IF_POLL_UP;
-	return eloop_timeout_add_msec(ifp->ctx->eloop, msec, if_poll, ifp);
-}
-
 /*
  * eth0.100:2 OR eth0i100:2 (seems to be NetBSD xvif(4) only)
  *
